UNIT KDS_Edit;
{+H
---------------------------------------------------------------------------
  File        - DSETEDIT.PAS
  Version     - 0.01
  Purpose     - Provide property editors for TKSS_DataSet components
  Author      - Keith S. Brown
                E-mail: keith_brown@juno.com
                http://www.ghg.net/chairouchu/
                2437 Bay Area Blvd #20, Houston TX / USA
  Copyright    1998 Keith S. Brown
  Restrictions- This component is FREEWARE!
                No warranty is given by the author, expressed or implied.
  Revised     - 1998.1228 (KSB) Simplified.
---------------------------------------------------------------------------}
INTERFACE
USES
  Windows,  Messages, SysUtils, Classes,  Graphics, Controls, Forms,    Dialogs,
  KDataSet, StdCtrls, Buttons,  ExtCtrls, KSSStore, Spin, Grids;

TYPE
  TEditColumnDefs  = CLASS(TForm)
    btnAdd    : TBitBtn;
    btnCancel : TBitBtn;
    btnDelete : TBitBtn;
    btnDn     : TBitBtn;
    btnOk     : TBitBtn;
    btnUp     : TBitBtn;
    cboxType  : TComboBox;
    edtName   : TEdit;
    edtWidth  : TSpinEdit;
    grdColumns: TStringGrid;

    Label1    : TLabel;
    Label2    : TLabel;
    Label3    : TLabel;
    Label4    : TLabel;
    Panel1    : TPanel;
    Panel2    : TPanel;
    Panel3    : TPanel;
    PROCEDURE   BtnAddClick(Sender: TObject);
    PROCEDURE   BtnDelClick(Sender: TObject);
    PROCEDURE   BtnDnClick(Sender: TObject);
    PROCEDURE   BtnSelectClick(Sender: TObject);
    PROCEDURE   BtnUpClick(Sender: TObject);
    PROCEDURE   BtnUpdateClick(Sender: TObject);
    PROCEDURE   FormClose(Sender: TObject; VAR Action: TCloseAction);
    PROCEDURE   FormCreate(Sender: TObject);
    PROCEDURE   FormShow(Sender: TObject);
    PROCEDURE   LinesAvailable(Sender: TObject);
    PROCEDURE   NewColumnDataAvailable(Sender: TObject);
  PRIVATE
    { Private declarations }
    FiMaxCol  : INTEGER;
    FOnClosDlg: TNotifyEvent;
    FOnShowDlg: TNotifyEvent;
    PROCEDURE   CopyFrom(aoHeader:TColumnDefs);
    PROCEDURE   CopyTo(aoHeader:TColumnDefs);
    PROCEDURE   SetMaxCol(aValue:INTEGER);
  PUBLIC
    { Public declarations }
    PROPERTY    MaxCols:INTEGER READ FiMaxCol WRITE SetMaxCol;
    PROPERTY    OnCloseDialog:TNotifyEvent READ FOnClosDlg WRITE FOnClosDlg;
    PROPERTY    OnShowDialog :TNotifyEvent READ FOnShowDlg WRITE FOnShowDlg;
  END {CLASS};


  TKSS_HeaderDialog= CLASS(TComponent)
  PRIVATE
    FoDataSet : TKSS_CustomDataSet;
    FOnClose  : TNotifyEvent;
    FOnShow   : TNotifyEvent;
  PROTECTED
    PROCEDURE   Notification(aComponent:TComponent; aOperation:TOperation); OVERRIDE;
  PUBLIC
    CONSTRUCTOR Create(aoOwner:TComponent); OVERRIDE;
    DESTRUCTOR  Destroy; OVERRIDE;
    FUNCTION    Execute:BOOLEAN;
  PUBLISHED
    PROPERTY    KDataSet:TKSS_CustomDataSet READ FoDataSet WRITE FoDataSet;
    PROPERTY    OnClose :TNotifyEvent READ FOnClose WRITE FOnClose;
    PROPERTY    OnShow  :TNotifyEvent READ FOnShow  WRITE FOnShow; 
  END {CLASS};


{}FUNCTION    EditHeaderInfo(aoHeader:TColumnDefs; aCaption:STRING):BOOLEAN;
              {=================================================}
IMPLEMENTATION
{$R *.DFM}


{}FUNCTION    EditHeaderInfo(aoHeader:TColumnDefs; aCaption:STRING):BOOLEAN;
{+O
---------------------------------------------------------------------------
  Remarks     - Used in KDSetBox's property editor.
---------------------------------------------------------------------------}
  VAR
    loDlg     : TEditColumnDefs;
  BEGIN
    loDlg  := TEditColumnDefs.Create(application);
    TRY
      loDlg.Caption := aCaption+loDlg.Caption;
      loDlg.CopyFrom(aoHeader);
      result := loDlg.ShowModal = mrOk;
      IF result THEN BEGIN
        loDlg.CopyTo(aoHeader);
      END {IF};
    FINALLY
      loDlg.Release;
    END {TRY};
{}END {EditHeaderInfo};




{}PROCEDURE   TEditColumnDefs.BtnAddClick(Sender: TObject);
{+O
---------------------------------------------------------------------------
  Purpose     - Add button event handler.
---------------------------------------------------------------------------}
  VAR
    li        : INTEGER;
    ls        : STRING;
  BEGIN
    ls := edtName.Text+','+cboxType.Text+','+IntToStr(edtWidth.Value);
    WITH grdColumns DO BEGIN
      li := RowCount;
      IF Cells[0,li-1] <> '' THEN BEGIN
        RowCount := Succ(RowCount);
        li := RowCount;
      END {IF};

      Cells[0,li-1] := ls;
    END {WITH};

    LinesAvailable(sender);
    IF edtName.Enabled THEN BEGIN
      edtName.SetFocus;
    END ELSE BEGIN
      grdColumns.SetFocus;
    END {IF};
{}END {BtnAddClick};




{}PROCEDURE   TEditColumnDefs.BtnDelClick(Sender: TObject);
{+O
---------------------------------------------------------------------------
  Purpose     - Delete button event handler.
---------------------------------------------------------------------------}
  VAR
    li        : INTEGER;
  BEGIN
    WITH grdColumns DO BEGIN
      Cells[0,Row] := '';
      FOR li := Row TO RowCount-2 DO BEGIN
        Rows[li] := Rows[li+1];
      END {FOR};

      RowCount := Pred(RowCount);
    END {WITH};

    LinesAvailable(sender);
{}END {DeleteLine};




{}PROCEDURE   TEditColumnDefs.BtnDnClick(Sender: TObject);
{+O
---------------------------------------------------------------------------
  Purpose     - Down button event handler.
---------------------------------------------------------------------------}
  VAR
    ls        : STRING;
  BEGIN
    WITH grdColumns DO BEGIN
      IF row < RowCount THEN BEGIN
        ls := Cells[0,row];
        Cells[0,row] := Cells[0,row+1];
        Cells[0,row+1] := ls;
        row := Succ(row);
      END {IF};
    END {WITH};
{}END {BtnDnClick};




{}PROCEDURE   TEditColumnDefs.BtnSelectClick(Sender: TObject);
{+O
---------------------------------------------------------------------------
  Purpose     - Select button event handler.
---------------------------------------------------------------------------}
  VAR
    ls        : STRING;
    ls1       : STRING;
  BEGIN
    WITH grdColumns DO BEGIN
      ls := Cells[0,row];
      ls1:= Copy(ls,1,Pos(',',ls)-1);
      edtName.Text := ls1;

      ls := Trim(Copy(ls,Pos(',',ls)+1,Length(ls)));
      ls1:= Copy(ls,1,Pos(',',ls)-1);
      cboxType.Text := ls1;

      ls := Trim(Copy(ls,Pos(',',ls)+1,Length(ls)));
      edtWidth.Value := StrToIntDef(ls,0);

      NewColumnDataAvailable(sender);
      edtName.SetFocus;
    END {WITH};
{}END {BtnSelectClick};




{}PROCEDURE   TEditColumnDefs.BtnUpClick(Sender: TObject);
{+O
---------------------------------------------------------------------------
  Purpose     - Up button event handler.
---------------------------------------------------------------------------}
  VAR
    ls        : STRING;
  BEGIN
    WITH grdColumns DO BEGIN
      IF row > 0 THEN BEGIN
        ls := Cells[0,row];
        Cells[0,row] := Cells[0,row-1];
        Cells[0,row-1] := ls;
        row := Pred(row);
      END {IF};
    END {WITH};
{}END {BtnUpClick};




{}PROCEDURE   TEditColumnDefs.BtnUpdateClick(Sender: TObject);
{+O
---------------------------------------------------------------------------
  Purpose     - Update button event handler.
---------------------------------------------------------------------------}
  BEGIN
    WITH grdColumns DO BEGIN
      Cells[0,row] := edtName.Text + ','+ cboxType.Text + ',' + IntToStr(edtWidth.Value);
    END {WITH};

    LinesAvailable(sender);
    IF edtName.Enabled THEN BEGIN
      edtName.SetFocus;
    END ELSE BEGIN
      grdColumns.SetFocus;
    END {IF};
{}END {BtnUpdateClick};




{}PROCEDURE   TEditColumnDefs.CopyFrom(aoHeader:TColumnDefs);
{+O
---------------------------------------------------------------------------
  Purpose     - Copy information from an existing header, so it can be displayed.
---------------------------------------------------------------------------}
  VAR
    lbMany    : BOOLEAN;
    lbOne     : BOOLEAN;
    li        : INTEGER;
  BEGIN
    WITH grdColumns DO BEGIN
      RowCount := 1;
      lbOne := FALSE;
      lbMany:= FALSE;
      Cells[0,0] := '';
      RowCount := aoHeader.Count;
      FOR li := 1 TO aoHeader.Count DO BEGIN
        Cells[0,li-1] := aoHeader.AsString[li-1];
        IF li = 1 THEN BEGIN
          lbOne := TRUE;
        END ELSE
        IF (li > 1) AND NOT lbMany THEN BEGIN
          lbMany := TRUE;
        END {IF};
      END {FOR};
    END {WITH};

    IF lbOne THEN BEGIN
      btnDelete.Enabled := TRUE;
    END {IF};

    IF lbMany THEN BEGIN
      btnUp.Enabled := TRUE;
      btnDn.Enabled := TRUE;
    END {IF};
{}END {CopyFrom};




{}PROCEDURE   TEditColumnDefs.CopyTo(aoHeader:TColumnDefs);
{+O
---------------------------------------------------------------------------
  Purpose     - Copy information to a header to update its content.
---------------------------------------------------------------------------}
  VAR
    li        : INTEGER;
    liCol     : INTEGER;
  BEGIN
    aoHeader.Clear;
    liCol := 0;
    WITH grdColumns DO BEGIN
      FOR li := 1 TO RowCount DO BEGIN
        IF Length(Trim(Cells[0,li-1])) > 0 THEN BEGIN
          aoHeader.AsString[liCol] := Cells[0,li-1];
          Inc(liCol);
        END {IF};
      END {FOR};
    END {WITH};
{}END {CopyTo};




{}PROCEDURE   TEditColumnDefs.FormClose(Sender: TObject; var Action: TCloseAction);
{+O
---------------------------------------------------------------------------
  Purpose     - Event handler. Close the editor form.
---------------------------------------------------------------------------}
  BEGIN
    IF Assigned(FOnClosDlg) THEN BEGIN
      FOnClosDlg(sender);
    END {IF};
{}END {FormClose};




{}PROCEDURE   TEditColumnDefs.FormCreate(Sender: TObject);
{+O
---------------------------------------------------------------------------
  Purpose     - Event handler. Create the editor form.
---------------------------------------------------------------------------}
  BEGIN
    btnAdd.Enabled := FALSE;
    btnDelete.Enabled := FALSE;
{}END {FormCreate};




{}PROCEDURE   TEditColumnDefs.FormShow(Sender: TObject);
{+O
---------------------------------------------------------------------------
  Purpose     - Event handler.
---------------------------------------------------------------------------}
  BEGIN
    IF FiMaxCol > 0 THEN BEGIN
      WITH btnDelete DO BEGIN
        Caption := '&Select';
        OnClick := BtnSelectClick;
      END {WITH};

      WITH btnAdd DO BEGIN
        Caption := 'u&Pdate';
        OnClick := BtnUpdateClick;
      END {WITH};

      btnUp.Enabled := FALSE;
      btnDn.Enabled := FALSE;
      cboxType.Enabled:= FALSE;
      Application.ProcessMessages;

      IF Assigned(FOnShowDlg) THEN BEGIN
        FOnShowDlg(sender);
      END {IF};
    END {IF};
{}END {FormShow};




{}PROCEDURE   TEditColumnDefs.LinesAvailable(Sender: TObject);
{+O
---------------------------------------------------------------------------
  Purpose     - Event handler. Enable/disable the up/down & delete buttons
---------------------------------------------------------------------------}
  BEGIN
    IF FiMaxCol = 0 THEN BEGIN
      IF grdColumns.RowCount > 1 THEN BEGIN
        btnUp.Enabled := TRUE;
        btnDn.Enabled := TRUE;
        btnDelete.Enabled := TRUE;
      END ELSE BEGIN
        btnUp.Enabled := FALSE;
        btnDn.Enabled := FALSE;
        btnDelete.Enabled := Length(grdColumns.Cells[0,0]) > 0;
      END {IF};
    END {IF};
{}END {LinesAvailable};




{}PROCEDURE   TEditColumnDefs.NewColumnDataAvailable(Sender: TObject);
{+O
---------------------------------------------------------------------------
  Purpose     - Event handler. Enable/disable the add button.
---------------------------------------------------------------------------}
  BEGIN
    IF (FiMaxCol > 0) THEN BEGIN
      LinesAvailable(sender);
    END {IF};

    IF (edtName.Text <> '') AND (edtWidth.Value <> 0) AND (cboxType.Text <> '') THEN BEGIN
      btnAdd.Enabled := TRUE;
    END ELSE BEGIN
      btnAdd.Enabled := FALSE;
    END {IF};
{}END {NewColumnDataAvailable};




{}PROCEDURE   TEditColumnDefs.SetMaxCol(aValue:INTEGER);
{+O
---------------------------------------------------------------------------
  Purpose     - Property access method.
---------------------------------------------------------------------------}
  BEGIN
    IF aValue >= 0 THEN BEGIN
      IF aValue <> FiMaxCol THEN BEGIN
        FiMaxCol := aValue;
      END {IF};
    END {IF};
{}END {SetMaxCol};




{}CONSTRUCTOR TKSS_HeaderDialog.Create(aoOwner:TComponent);
{+O
---------------------------------------------------------------------------
  Purpose     - Create the component.
---------------------------------------------------------------------------}
  BEGIN
    INHERITED Create(aoOwner);
    FoDataSet:= NIL;
{}END {Create};




{}DESTRUCTOR  TKSS_HeaderDialog.Destroy;
{+O
---------------------------------------------------------------------------
  Purpose     - Release auxiliary memory.
---------------------------------------------------------------------------}
  BEGIN
    INHERITED Destroy;
{}END {Destroy};




TYPE
  TKDS        = CLASS(TKSS_CustomDataSet);

{}FUNCTION    TKSS_HeaderDialog.Execute:BOOLEAN;
{+O
---------------------------------------------------------------------------
  Purpose     - Create & configure the form used as a dialog box. If the
                user clicked on "OK" then update the header information
                in the linked dataset.
---------------------------------------------------------------------------}
  VAR
    loForm    : TEditColumnDefs;
    loHdr     : TColumnDefs;
  BEGIN
    result := Assigned(FoDataSet);
    IF result THEN BEGIN
      loHdr := TColumnDefs.Create;
      TRY
        loHdr.CopyFrom(TKDS(FoDataSet).Data.Header);
        loForm := TEditColumnDefs.Create(Application);
        TRY
          loForm.MaxCols := TKDS(FoDataSet).Data.ColCount;
          loForm.CopyFrom(loHdr);
          loForm.OnCloseDialog := FOnClose;
          loForm.OnShowDialog  := FOnShow;
          result := loForm.ShowModal = mrOk;
          IF result THEN BEGIN
            loForm.CopyTo(loHdr);
            TKDS(FoDataSet).Data.Header.CopyFrom(loHdr);
            TKDS(FoDataSet).RefreshDisplayLabels;
          END {IF};
        FINALLY
          loForm.Release;
        END {TRY};
      FINALLY
        loHdr.Free;
      END {TRY};
    END {IF};
{}END {Execute};




{}PROCEDURE   TKSS_HeaderDialog.Notification(aComponent:TComponent; aOperation:TOperation);
{+O
---------------------------------------------------------------------------
  Purpose     - Disconnect the associated DataSet if it has been removed
                from the form.
---------------------------------------------------------------------------}
  BEGIN
    INHERITED Notification(aComponent,aOperation);
    IF (aOperation = opRemove) AND (aComponent = FoDataSet) THEN BEGIN
      FoDataSet := NIL;
    END {IF};
{}END {Notification};


END {DSetEdit}.
